require('colors');

var readline  = require('readline')
  , fs        = require('fs')
  , utilities = require('utilities')
  , Loader    = require('./Loader').Loader
  , loader    = new Loader(__dirname)
  , assert    = require('assert')
  , url       = require('url')

const PATH_REGEX = /\.?([a-z|0-9|_]+)\.?/gi;
module.exports = utilities.mixin(utilities, loader.load().utilities);

module.exports.assert     = assert;
module.exports.url        = url;
module.exports.noop       = function() {};
module.exports.returnMe   = function(){ return this; };
module.exports.makeArray  = function(object) {
  if (! object.length) {
    object.length = 0;

    for (var prop in object) {
      if (! isNaN(+prop)){
        ++object.length;
      }
    }
  }

  return [].map.call(object, function(v){ return v; });
};

module.exports.PATH_REGEX = PATH_REGEX;
module.exports.valueFromPath = function(path, object){
  var value

  path  = path.replace(PATH_REGEX,'["$1"]');
  value =  (new Function('o',[
    'try {',
      'return o' + path,
    '} catch(e) {',
      'return false;',
    '}'
  ].join('\n')))(object);

  return value;
}

module.exports.setValueFromPath = function(path, value, object) {
  path    = path.replace(this.PATH_REGEX, '["$1"]');
  object  = (new Function('o', 'v' ,[
    'try {',
      'o' + path + ' = v;',
      'return o;',
    '} catch(e) {',
      'return false;',
    '}'
  ].join('\n')))(object, value);

  return object;
};


module.exports.parseStringVariables = function(string, vars) {
  var key, regex

  for (key in vars) {
    regex  = new RegExp('\\$\\{' + key + '\\}', 'g');
    string = string.replace(regex, vars[key]); // Set the vars
  }

  string = string.replace(/(\$\{[a-z]+\})/gi,''); // Get rid of any undefined

  return string
}

module.exports.appRequire = function(path, logging) {
  logging = logging === undefined? true : logging;
  logging = IS_EXECUTED_FROM_BIN? logging : false
  var msg = "Requiring ".cyan + path

  if (logging !== false) {
    console.log(msg);
  }
  
  return require(APP_PATH + path);
}

module.exports.appStat = function(path) {
  if (APP_PATH !== PINEAPPLE_PATH && IS_EXECUTED_FROM_BIN) {
    return this.stat(APP_PATH + path);
  }
  else {
    return false;
  }
}

module.exports.CliInterface = function CliInterface(input, output, completetions) {
  var self = this

  this.input          = input;
  this.output         = output;
  this.completetions  = completetions || [];
  this.interface      = readline.createInterface({
    input     : this.input,
    output    : this.output,
    completer : (this.completetions.length? function(line) {
                    var completions = self.completions.map(function(v){ return "." + v })
                      , hits        = completions.filter(function(c) { return c.indexOf(line) == 0 })
                    
                    // show all completions if none found
                    return [hits.length ? hits : completions, line]
                  } 
                : false)
  });

  this.setPrompt = function(prompt) {
    this.interface.setPrompt(prompt, prompt.length || 0);

    return this;
  }

  this.question = function() {    
    this.interface.question.apply(this.interface, arguments);

    return this;
  }

  this.close = function() {
    this.interface.close();
    return this;
  }

  this.resume = function() {
    this.interface.resume();

    return this;
  }

  this.pause = function() {
    this.interface.pause();

    return this;
  }
};

module.exports.object.clone = function(object) {
  return (function(){ 
    return function (obj) { Clone.prototype=obj; return new Clone() };
    function Clone(){}
  }())(object);
};

module.exports.unixTimestamp = function() {
  return Math.round(Date.now()/1000);
};